package net.parsell.cherry.core;

import com.google.common.collect.ImmutableMap.Builder;

import net.fabricmc.fabric.api.blockrenderlayer.v1.BlockRenderLayerMap;
import net.fabricmc.fabric.api.client.rendering.v1.ColorProviderRegistry;
import net.fabricmc.fabric.api.object.builder.v1.block.FabricBlockSettings;
import net.fabricmc.fabric.api.registry.FlammableBlockRegistry;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.Blocks;
import net.minecraft.block.FenceBlock;
import net.minecraft.block.FenceGateBlock;
import net.minecraft.block.LeavesBlock;
import net.minecraft.block.PillarBlock;
import net.minecraft.block.SlabBlock;
import net.minecraft.block.StairsBlock;
import net.minecraft.client.render.RenderLayer;
import net.minecraft.util.Identifier;
import net.minecraft.util.registry.Registry;
import net.parsell.cherry.common.blocks.*;
import net.parsell.cherry.mixin.AxeItemAccessor;

public class CherryBlocks {
	private static Builder<Block, Block> STRIPPABLE_BLOCKS = new Builder<Block, Block>();

	public static final Block CHERRYLOGSTRIPPED = registerPillar("stripped_cherry_log", FabricBlockSettings.copyOf(Blocks.STRIPPED_OAK_LOG), 5, 5);
	public static final Block CHERRYLOG = registerPillar("cherry_log", FabricBlockSettings.copyOf(Blocks.OAK_LOG), 5, 5, CHERRYLOGSTRIPPED);
	public static final Block CHERRYWOOD = registerPillar("cherry_wood", FabricBlockSettings.copyOf(Blocks.OAK_WOOD), 5, 5);
	public static final Block CHERRYWOODSTRIPPED = registerPillar("stripped_cherry_wood", FabricBlockSettings.copyOf(Blocks.STRIPPED_OAK_WOOD), 5, 5);
	public static final Block CHERRYLEAVES = registerLeaves("cherry_leaves", FabricBlockSettings.copyOf(Blocks.OAK_LEAVES), 30, 60, 6649929);
	public static final Block CHERRYPLANKS = registerBlock("cherry_planks", FabricBlockSettings.copyOf(Blocks.OAK_PLANKS), 5, 20);
	public static final Block CHERRYFENCE = registerFence("cherry_fence", FabricBlockSettings.copyOf(Blocks.OAK_FENCE), 5, 20); // TODO: Textures, tags (as required)
	public static final Block CHERRYFENCEGATE = registerFenceGate("cherry_fence_gate", FabricBlockSettings.copyOf(Blocks.OAK_FENCE_GATE), 5, 20); // Textures, recipe, loot_tables, tags (as required)
	public static final Block CHERRYDOOR = registerDoor("cherry_door", FabricBlockSettings.copyOf(Blocks.OAK_DOOR), 5, 20); // TODO: recipe, loot_tables, tags (as required)
	public static final Block CHERRYTRAPDOOR = registerTrapDoor("cherry_trapdoor", FabricBlockSettings.copyOf(Blocks.OAK_TRAPDOOR), 5, 20); //TODO: recipe, loot_tables, tags (as required)
	public static final Block CHERRYSLAB = registerSlab("cherry_slab", FabricBlockSettings.copyOf(Blocks.OAK_SLAB), 5, 20); // TODO: recipe, loot tables, tags (as required)
	public static final Block CHERRYSTAIRS = registerStair("cherry_stairs", CHERRYPLANKS.getDefaultState(), FabricBlockSettings.copyOf(Blocks.OAK_STAIRS), 5, 20); //TODO: recipe, loot tables, tags (as required)

	private static Block registerBlock(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new Block(settings);
		Registry.register(Registry.BLOCK, new Identifier("cherry", blockID), block);
		FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
		return block;
	}

	private static Block registerSlab(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new SlabBlock(settings);
		Registry.register(Registry.BLOCK, new Identifier("cherry", blockID), block);
		FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
		return block;
	}

	private static Block registerStair(String blockID, BlockState baseBlockState, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new CherryStairsBlock(baseBlockState, settings);
		Registry.register(Registry.BLOCK, new Identifier("cherry", blockID), block);
		//FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
		return block;
	}

	private static Block registerFence(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new FenceBlock(settings);
		Registry.register(Registry.BLOCK, new Identifier("cherry", blockID), block);
		FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
		return block;
	}

	private static Block registerFenceGate(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new FenceGateBlock(settings);
		Registry.register(Registry.BLOCK, new Identifier("cherry", blockID), block);
		FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
		return block;
	}

	private static Block registerPillar(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread){
		Block block = new PillarBlock(settings);
		Registry.register(Registry.BLOCK, new Identifier("cherry", blockID), block);
		FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
		return block;
	}

	private static Block registerPillar(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread, Block strippedVariant){
		Block block = new PillarBlock(settings);
		Registry.register(Registry.BLOCK, new Identifier("cherry", blockID), block);
		FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
		STRIPPABLE_BLOCKS.put(block, strippedVariant);
		return block;
	}

	private static Block registerLeaves(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread, int color){
		Block block = new LeavesBlock(settings);
		Registry.register(Registry.BLOCK, new Identifier("cherry", blockID), block);
		FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
		ColorProviderRegistry.BLOCK.register((state, world, pos, tintIndex) -> {
			return color;
		 }, block);
		return block;
	}

	private static Block registerDoor(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread){
		Block block = new CherryDoorBlock(settings);
		Registry.register(Registry.BLOCK, new Identifier("cherry", blockID), block);
		FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
		BlockRenderLayerMap.INSTANCE.putBlock(block, RenderLayer.getCutout());
		return block;
	}

	private static Block registerTrapDoor(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread){
		Block block = new CherryTrapdoorBlock(settings);
		Registry.register(Registry.BLOCK, new Identifier("cherry", blockID), block);
		FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
		BlockRenderLayerMap.INSTANCE.putBlock(block, RenderLayer.getCutout());
		return block;
	}

	public static void init(){
		// Ensure not to overwrite on previous strippable logs
		AxeItemAccessor.getStrippedLogs().forEach((log, strippedLog) -> {
			STRIPPABLE_BLOCKS.put(log, strippedLog);
		});

		// Register unique logs
		AxeItemAccessor.setStrippedLogs(STRIPPABLE_BLOCKS.build());
	}
}